'use strict';

var _core = require('@cycle/core');

var _rx = require('rx');

var _rx2 = _interopRequireDefault(_rx);

var _restartable = require('./restartable');

var _restartable2 = _interopRequireDefault(_restartable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function restart(main, drivers, _ref) {
  var sources = _ref.sources;
  var sinks = _ref.sinks;
  var isolate = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];
  var timeToTravelTo = arguments.length <= 4 || arguments[4] === undefined ? null : arguments[4];

  sources.dispose();
  sinks && sinks.dispose();

  if (typeof isolate === 'function' && 'reset' in isolate) {
    isolate.reset();
  }

  for (var driverName in drivers) {
    var driver = drivers[driverName];

    if (driver.onPreReplay) {
      driver.onPreReplay();
    }
  }

  var newSourcesAndSinks = (0, _core.run)(main, drivers);

  setTimeout(function () {
    var scheduler = new _rx2.default.HistoricalScheduler();

    for (var driverName in drivers) {
      var driver = drivers[driverName];

      if (driver.replayLog) {
        var log$ = sources[driverName].log$;

        driver.replayLog(scheduler, log$, timeToTravelTo);
      }
    }

    scheduler.scheduleAbsolute({}, new Date(), function () {
      // TODO - remove this setTimeout, figure out how to tell when an async app is booted
      setTimeout(function () {
        for (var driverName in drivers) {
          var driver = drivers[driverName];

          if (driver.onPostReplay) {
            driver.onPostReplay();
          }
        }
      }, 500);
    });

    scheduler.start();
  }, 1);

  return newSourcesAndSinks;
}

function rerunner(run, isolate) {
  var sourcesAndSinks = {};
  var first = true;
  return function (main, drivers) {
    var timeToTravelTo = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];

    if (first) {
      sourcesAndSinks = run(main, drivers);
      first = false;
    } else {
      sourcesAndSinks = restart(main, drivers, sourcesAndSinks, isolate, timeToTravelTo);
    }
    return sourcesAndSinks;
  };
}

module.exports = {
  restart: restart,
  restartable: _restartable2.default,
  rerunner: rerunner
};